# coding: utf-8
u"""Классы-примеси для моделей Django."""
from django.db import models


class DeleteAfterSaveMixin(models.Model):

    u"""Примесь для удаления объектов после сохранения модели.

    Функционал данной примеси актуален при валидации моделей. В некоторых
    случаях во время валидации выясняется, что какой-либо зависимый объект
    должен быть удален. Но т.к. во время валидации экземляра модели удалять
    объект еще рано, то можно этот объект пометить для удаления после
    сохранения экземпляра модели.

    Рекомендуется указывать данную примесь первой в списке базовых классов
    модели, тогда удаление объектов будет выполняться в последнюю очередь.
    """

    def __init__(self, *args, **kwargs):
        super(DeleteAfterSaveMixin, self).__init__(*args, **kwargs)

        # Список объектов, подлежащих удалению после сохранения self
        self.__objects_for_delete = set()

    def delete_after_save(self, obj):
        u"""Удаляет указанный объект после сохранения.

        Указанный в аргументе obj объект помещается в список объектов, которые
        будут удалены после сохранения.

        :param obj: Объект модели, подлежащий удалению после сохранения.
        :type obj: django.db.models.Model
        """
        if obj is None or self in self.__objects_for_delete:
            return

        assert isinstance(obj, models.Model), type(obj)

        self.__objects_for_delete.add(obj)

    def save(self, *args, **kwargs):
        super(DeleteAfterSaveMixin, self).save(*args, **kwargs)

        while self.__objects_for_delete:
            obj = self.__objects_for_delete.pop()
            if obj.pk is not None and obj != self:
                obj.safe_delete()

    class Meta:
        abstract = True


class StringFieldsCleanerMixin(models.Model):

    u"""Примесь для удаления из строковых полей модели лишних пробелов.

    Во всех текстовых полях модели удаляет пробельные символы в начале и конце
    строки, несколько идущих подряд пробелов заменяет на один.

    Для полей с разрешенными пустыми значениями (null=True) пустые строки
    заменяет на None.

    .. note::
       В документации Django текстовых полей не рекомендуется использовать
       значение *None*, т.к. в этом случае возникает неоднозначность - пустым
       значением будет являться не только None, но и пустая строка. Подробнее
       см. http://djbook.ru/rel1.4/ref/models/fields.html#null. Но
       использование пустой строки совместно с ограничением уникальности
       приводит к невозможности сохранения более одной записи с пустым
       значением, поэтому для однозначности в текстовых полях будем
       использовать значение None, если указано null=True, и пустую строку в
       остальных случаях.
    """

    def clean_fields(self, exclude=None):
        for field in self._meta.fields:
            if isinstance(field, (models.TextField, models.CharField)):
                field_value = getattr(self, field.attname)

                if field_value is not None:
                    field_value = unicode(field_value).strip()
                    # Удаление лишних пробелов
                    while field_value.find(u'  ') != -1:
                        field_value = field_value.replace(u'  ', u'')

                if not field_value:
                    field_value = None if field.null else u''

                setattr(self, field.attname, field_value)

        return super(StringFieldsCleanerMixin, self).clean_fields(exclude)

    class Meta:
        abstract = True
